#include <stdio.h>
#include <stdlib.h>
#include <types.h>
#include <globals.h>
#include <utils.h>
#include <latinit.h>
#include <latrand.h>
#include <spinor_zaxpy.h>

#define SEED 1771

void
usage(char *argv[])
{
  fprintf(stderr, " Usage: %s LENGTH NITERS\n", argv[0]);
  return;
}

int
main(int argc, char *argv[])
{
  if(argc != 3) {
    usage(argv);
    exit(1);
  }

  length = atol(argv[1]);
  unsigned long int niters = atol(argv[2]);
 
  complex *x = spinor_init(length);
  complex *y = spinor_init(length);
  complex a;

  init_rand(SEED);
  spinor_rand(x);
  spinor_rand(y);
  a = scalar_rand();

  double t = stop_watch(0);
  for(int i=0; i<niters; i++)
    spinor_zaxpy(a, x, y);
  t = stop_watch(t);
  
  printf(" [ %2d ]: %8d iters, length = %8d, t = %10.2lf usec per iter, BW = %6.2lf GB/s\n", sizeof(a), niters, length, 
	 t/(double)niters, length*3*NS*NC*sizeof(a)/t*niters*1.e6/1024/1024/1024);
  
  free(x);
  free(y);
  return 0;
}
