#include <stdio.h>
#include <stdlib.h>
#include <types.h>
#include <globals.h>
#include <utils.h>
#include <latinit.h>
#include <latrand.h>
#include <mul_su3_spinor.h>

#define SEED 1771

void
usage(char *argv[])
{
  fprintf(stderr, " Usage: %s LENGTH NITERS\n", argv[0]);
  return;
}

int
main(int argc, char *argv[])
{
  if(argc != 3) {
    usage(argv);
    exit(1);
  }

  length = atol(argv[1]);
  unsigned long int niters = atol(argv[2]);
 
  complex *x = spinor_init(length);
  complex *y = spinor_init(length);
  complex *u = su3_init(length); 

  init_rand(SEED);
  spinor_rand(x);
  su3_rand(u);
  
  double t = stop_watch(0);
  for(int i=0; i<niters; i++)
    mul_su3_spinor(y, u, x);
  t = stop_watch(t);
  
  printf(" 0: %8d iters, length = %8d, t = %10.2lf usec per iter, BW = %6.2lf GB/s, perf = %6.2lf Mflop/s\n", niters, length, 
	 t/(double)niters, (2*NS*NC+NC*NC)*sizeof(complex)*1.e6*length*niters/t/1024/1024/1024, 66*(double)niters*NS*(double)length/t);

  free(x);
  free(y);
  free(u);
  return 0;
}
