#include <stdio.h>
#include <stdlib.h>
#include <types.h>
#include <globals.h>
#include <utils.h>
#include <latinit.h>
#include <latrand.h>
#include <mul_su3_spinor.h>

#define SEED 1771

void
usage(char *argv[])
{
  fprintf(stderr, " Usage: %s LENGTH NITERS\n", argv[0]);
  return;
}

double
diff(complex *x, complex *y)
{
  double sum = 0;
  double nrm = 0;
  for(int i=0; i<NS*NC*length; i++) {
    double dr = x[i].re - y[i].re;
    double di = x[i].im - y[i].im;
    double sr = x[i].re + y[i].re;
    double si = x[i].im + y[i].im;
    sum += (dr*dr + di*di);
    nrm += (sr*sr + si*si);
  }
  return sum/nrm;
}

int
main(int argc, char *argv[])
{
  if(argc != 3) {
    usage(argv);
    exit(1);
  }

  length = atol(argv[1]);
  unsigned long int niters = atol(argv[2]);
 
  complex *x = spinor_init(length);
  complex *y = spinor_init(length);
  complex *z = spinor_init(length);
  complex *u = su3_init(length); 

  init_rand(SEED);
  spinor_rand(x);
  su3_rand(u);
  
  mul_su3_spinor(y, u, x);
  double t = stop_watch(0);
  for(int i=0; i<niters; i++)
    mul_su3_spinor(y, u, x);
  t = stop_watch(t);
  
  printf(" 0: %8d iters, length = %8d, t = %10.2lf usec per iter, BW = %6.2lf GB/s, perf = %6.2lf Mflop/s\n", niters, length, 
	 t/(double)niters, (2*NS*NC+NC*NC)*(sizeof(double)*2*length*niters*1e6)/t/1024/1024/1024, 66*(double)niters*NS*(double)length/t);

  mul_su3_spinor_intrins(z, u, x);
  t = stop_watch(0);
  for(int i=0; i<niters; i++)
    mul_su3_spinor_intrins(z, u, x);
  t = stop_watch(t);
  
  printf(" 1: %8d iters, length = %8d, t = %10.2lf usec per iter, BW = %6.2lf GB/s, perf = %6.2lf Mflop/s\n", niters, length, 
	 t/(double)niters, (2*NS*NC+NC*NC)*(sizeof(double)*2*length*niters*1e6)/t/1024/1024/1024, 66*(double)niters*NS*(double)length/t);

  double d = diff(y,z);
  printf(" Diff = %e\n", d);

  free(x);
  free(y);
  free(z);
  free(u);
  return 0;
}
