#include <stdlib.h>
#include <utils.h>
#include <types.h>
#include <globals.h>
#include <xmmintrin.h>

#ifdef SINGLE_PRECISION
void
spinor_zaxpy(complex a, complex *x, complex *y)
{
  __m128 ar __attribute__((aligned(16))) = { a.re, a.re, a.re, a.re};
  __m128 ai __attribute__((aligned(16))) = {-a.im, a.im,-a.im, a.im};

  __m128 register a0 = _mm_load_ps((float *)&ar);
  __m128 register a1 = _mm_load_ps((float *)&ai);

  /*
   * TODO: Add an omp pragma to parallelize this for-loop
   */
#pragma omp parallel for firstprivate(a0,a1,x,y)
  for(int v=0; v<length*NC*NS; v+=2) {
    __m128 register rx0, rx1, ry0;
    
    rx0 = _mm_load_ps((float *)&x[v]);
    rx1 = _mm_shuffle_ps(rx0, rx0, _MM_SHUFFLE(2,3,0,1));
    ry0 = _mm_load_ps((float *)&y[v]);

    rx0 = _mm_mul_ps(rx0, a0);
    ry0 = _mm_add_ps(rx0, ry0);
    rx1 = _mm_mul_ps(rx1, a1);
    ry0 = _mm_add_ps(rx1, ry0);

    _mm_store_ps((float *)&y[v], ry0);
  }
  return;
}
#else
void
spinor_zaxpy(complex a, complex *x, complex *y)
{
  __m128d ar __attribute__((aligned(16))) = { a.re, a.re};
  __m128d ai __attribute__((aligned(16))) = {-a.im, a.im};

  __m128d register a0 = _mm_load_pd((double *)&ar);
  __m128d register a1 = _mm_load_pd((double *)&ai);

  /*
   * TODO: Add an omp pragma to parallelize this for-loop
   */
#pragma omp parallel for firstprivate(a0,a1,x,y)
  for(int v=0; v<length*NC*NS; v++) {
    __m128d register rx0, rx1, ry0;
    
    rx0 = _mm_load_pd((double *)&x[v]);
    rx1 = _mm_shuffle_pd(rx0, rx0, _MM_SHUFFLE2(0,1));
    ry0 = _mm_load_pd((double *)&y[v]);

    rx0 = _mm_mul_pd(rx0, a0);
    ry0 = _mm_add_pd(rx0, ry0);
    rx1 = _mm_mul_pd(rx1, a1);
    ry0 = _mm_add_pd(rx1, ry0);
    
    _mm_store_pd((double *)&y[v], ry0);
  }
  return;
}
#endif
