#include <stdlib.h>
#include <utils.h>
#include <types.h>
#include <globals.h>
#include <xmmintrin.h>

void
mul_su3_spinor(complex *y, complex *u, complex *x)
{
#pragma omp parallel for firstprivate(y,u,x)
  for(int v=0; v<length; v++)
    for(int s=0; s<NS; s++)
      for(int c0=0; c0<NC; c0++) {
	complex y0 = (complex){0., 0.};
	complex *u0 = &u[v*NC*NC + c0*NC];
	complex *x0 = &x[v*NS*NC + s*NC];
	for(int c1=0; c1<NC; c1++) {
	  y0.re += u0[c1].re*x0[c1].re - u0[c1].im*x0[c1].im;
	  y0.im += u0[c1].re*x0[c1].im + u0[c1].im*x0[c1].re;
	}
	y[v*NS*NC + s*NC + c0] = y0;
      }
  return;
}

void
mul_su3_spinor_intrins(complex *y, complex *u, complex *x)
{
#pragma omp parallel for firstprivate(u,y,x)
  for(int i=0; i<length; i++) {
    double __attribute__((aligned(16))) s[2] = {-1.0, 1.0};
    __m128d sign = _mm_load_pd(s);
    __m128d register  g00 = _mm_load_pd(&u[0 + NC*NC*i].re);
    __m128d register  g01 = _mm_load_pd(&u[1 + NC*NC*i].re);
    __m128d register  g02 = _mm_load_pd(&u[2 + NC*NC*i].re);

    __m128d register  g10 = _mm_load_pd(&u[3 + NC*NC*i].re);
    __m128d register  g11 = _mm_load_pd(&u[4 + NC*NC*i].re);
    __m128d register  g12 = _mm_load_pd(&u[5 + NC*NC*i].re);

    __m128d register  g20 = _mm_load_pd(&u[6 + NC*NC*i].re);
    __m128d register  g21 = _mm_load_pd(&u[7 + NC*NC*i].re);
    __m128d register  g22 = _mm_load_pd(&u[8 + NC*NC*i].re);

    for(int sp=0; sp<NS; sp++) {
      __m128d register  x0 = _mm_load_pd(&x[0 + NC*sp + NC*NS*i].re);
      __m128d register  x1 = _mm_load_pd(&x[1 + NC*sp + NC*NS*i].re);
      __m128d register  x2 = _mm_load_pd(&x[2 + NC*sp + NC*NS*i].re);

      __m128d register  ix0 = _mm_shuffle_pd(x0, x0, _MM_SHUFFLE2(0, 1)); /* swap re with im */
      __m128d register  ix1 = _mm_shuffle_pd(x1, x1, _MM_SHUFFLE2(0, 1)); /* swap re with im */
      __m128d register  ix2 = _mm_shuffle_pd(x2, x2, _MM_SHUFFLE2(0, 1)); /* swap re with im */

      __m128d register  y0r, y0i, y0;
      __m128d register  y1r, y1i, y1;
      __m128d register  y2r, y2i, y2;
      __m128d register  a0, a1, a2, a3;

      y0r = _mm_mul_pd(g00, x0);
      a0 = _mm_mul_pd(g01, x1);
      y0r = _mm_add_pd(a0, y0r);
      a0 = _mm_mul_pd(g02, x2);
      y0r = _mm_add_pd(a0, y0r);

      y0i = _mm_mul_pd(g00, ix0);
      a0 = _mm_mul_pd(g01, ix1);
      y0i = _mm_add_pd(a0, y0i);
      a0 = _mm_mul_pd(g02, ix2);
      y0i = _mm_add_pd(a0, y0i);

      /* */
      y1r = _mm_mul_pd(g10, x0);
      a0 = _mm_mul_pd(g11, x1);
      y1r = _mm_add_pd(a0, y1r);
      a0 = _mm_mul_pd(g12, x2);
      y1r = _mm_add_pd(a0, y1r);

      y1i = _mm_mul_pd(g10, ix0);
      a0 = _mm_mul_pd(g11, ix1);
      y1i = _mm_add_pd(a0, y1i);
      a0 = _mm_mul_pd(g12, ix2);
      y1i = _mm_add_pd(a0, y1i);
          
      /* */
      y2r = _mm_mul_pd(g20, x0);
      a0 = _mm_mul_pd(g21, x1);
      y2r = _mm_add_pd(a0, y2r);
      a0 = _mm_mul_pd(g22, x2);
      y2r = _mm_add_pd(a0, y2r);
      
      y2i = _mm_mul_pd(g20, ix0);
      a0 = _mm_mul_pd(g21, ix1);
      y2i = _mm_add_pd(a0, y2i);
      a0 = _mm_mul_pd(g22, ix2);
      y2i = _mm_add_pd(a0, y2i);
      
      a0 = _mm_shuffle_pd(y0r, y0i, _MM_SHUFFLE2(0, 0));
      a1 = _mm_shuffle_pd(y0r, y0i, _MM_SHUFFLE2(1, 1));
      y0 = _mm_mul_pd(a1, sign);
      y0 = _mm_add_pd(a0, y0);

      a0 = _mm_shuffle_pd(y1r, y1i, _MM_SHUFFLE2(0, 0));
      a1 = _mm_shuffle_pd(y1r, y1i, _MM_SHUFFLE2(1, 1));
      y1 = _mm_mul_pd(a1, sign);
      y1 = _mm_add_pd(a0, y1);

      a0 = _mm_shuffle_pd(y2r, y2i, _MM_SHUFFLE2(0, 0));
      a1 = _mm_shuffle_pd(y2r, y2i, _MM_SHUFFLE2(1, 1));
      y2 = _mm_mul_pd(a1, sign);
      y2 = _mm_add_pd(a0, y2);

      _mm_stream_pd(&y[0 + NC*sp + NC*NS*i].re, y0);
      _mm_stream_pd(&y[1 + NC*sp + NC*NS*i].re, y1);      
      _mm_stream_pd(&y[2 + NC*sp + NC*NS*i].re, y2);      
    }
  }
  return;
}
