#include <defs.h>

void
lapl_iter(float *out, float sigma, float *in)
{
#pragma omp parallel firstprivate(out, in)
  {
  float delta = sigma / (1+4*sigma);
  float norm = 1./(1+4*sigma);
  /* Do lapl iteration on volume */
#pragma omp for
  for(int y=1; y<Ly-1; y++)
    for(int x=1; x<Lx-1; x++) {
      int v00 = x+y*Lx;
      int v0p = y*Lx + x+1;
      int v0m = y*Lx + x-1;
      int vp0 = v00 + Lx;
      int vm0 = v00 - Lx;
      out[v00] = norm*in[v00] +
	delta*(in[v0p] + in[v0m] + in[vp0] + in[vm0]);
    }
  
#pragma omp for 
  for(int x=0; x<Lx; x++) {
    int y = 0;
    int v00 = x;
    int v0p = (x+1)%Lx;
    int v0m = (Lx+x-1)%Lx;
    int vp0 = x + Lx;
    int vm0 = x + (Ly-1)*Lx;
    out[v00] = norm*in[v00] +
      delta*(in[v0p] + in[v0m] + in[vp0] + in[vm0]);
  }
#pragma omp for 
  for(int x=0; x<Lx; x++) {
    int y = Ly-1;
    int v00 = x+y*Lx;
    int v0p = (x+1)%Lx + y*Lx;
    int v0m = (Lx+x-1)%Lx + y*Lx;
    int vp0 = x;
    int vm0 = v00 - Lx;
    out[v00] = norm*in[v00] +
      delta*(in[v0p] + in[v0m] + in[vp0] + in[vm0]);
  }
#pragma omp for 
  for(int y=1; y<Ly-1; y++) {
    int x = 0;
    int v00 = y*Lx;
    int v0p = v00 + 1;
    int v0m = v00 + Lx - 1;
    int vp0 = v00 + Lx;
    int vm0 = v00 - Lx;
    out[v00] = norm*in[v00] +
      delta*(in[v0p] + in[v0m] + in[vp0] + in[vm0]);
  }
#pragma omp for 
  for(int y=1; y<Ly-1; y++) {
    int x = Lx-1;
    int v00 = y*Lx+x;
    int v0p = y*Lx;
    int v0m = v00-1;
    int vp0 = v00+Lx;
    int vm0 = v00-Lx;
    out[v00] = norm*in[v00] +
      delta*(in[v0p] + in[v0m] + in[vp0] + in[vm0]);
  }
  }
  return;
}
