#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <defs.h>
#include <io.h>
#include <lapl_ss.h>
#include <lapl.h>
#include <sys/time.h>

double
stop_watch(double t0) 
{
  double time;
  struct timeval t;
  gettimeofday(&t, NULL);
  time = (double) t.tv_sec + (double) t.tv_usec * 1e-6;  
  return time-t0;
}

void
usage(char *argv[]) {
  fprintf(stderr, " Usage: %s LX LY NITER IN_FILE OUT_FILE\n", argv[0]);
  return;
}

int
main(int argc, char *argv[]) {
  /* Check the number of command line arguments */
  if(argc != 6) {
    usage(argv);
    exit(1);
  }
  /* The length of the array in x and y is read from the command
     line */
  Lx = atoi(argv[1]);
  Ly = atoi(argv[2]);
  /* The number of iterations */
  int niter = atoi(argv[3]);
  /* Fixed "sigma" */
  float sigma = 0.01;
  printf(" Ly,Lx = %d,%d\n", Ly, Lx);
  printf(" niter = %d\n", niter);
  printf(" input file = %s\n", argv[4]);
  printf(" output file = %s\n", argv[5]);
  /* Allocate the buffer for the data */
  float *arr_in = malloc(sizeof(float)*Lx*Ly);
  float *arr[2] = {malloc(sizeof(float)*Lx*Ly),
		   malloc(sizeof(float)*Lx*Ly)};
  /* read file to buffer */
  read_from_file(arr_in, argv[4]);
  lapl_iter(arr[0], sigma, arr_in);
  double t0 = stop_watch(0);
  for(int i=0; i<niter; i++) {
    lapl_iter(arr[(i+1)%2], sigma, arr[i%2]);
  }
  t0 = stop_watch(t0)/(double)niter;
  /* write the result after niter iterations */
  /* construct filename */
  char fname[256];
  sprintf(fname, "%s.%08d", argv[5], niter);
  /* write to file */
  // write_to_file(fname, arr[niter%2]);
  /* write timing info */
  printf(" 0: iters = %8d, (Lx,Ly) = %6d, %6d, t = %8.1f usec/iter, BW = %6.3f GB/s, P = %6.3f Gflop/s\n",
	 niter, Lx, Ly, t0*1e6, 
	 Lx*Ly*sizeof(float)*2.0/(t0*1.0e9), 
	 (Lx*Ly*6.0)/(t0*1.0e9));

  /* allocate super-site buffers */
  supersite *ssarr[2];
  posix_memalign((void**)&ssarr[0], 16, sizeof(supersite)*Lx*Ly/4);
  posix_memalign((void**)&ssarr[1], 16, sizeof(supersite)*Lx*Ly/4);
  /* convert input array to super-site packed */
  to_supersite(ssarr[0], arr_in);
  lapl_iter_supersite(ssarr[1], sigma, ssarr[0]);
  /* do iterations, record time */
  t0 = stop_watch(0);
  for(int i=0; i<niter; i++) {
    lapl_iter_supersite(ssarr[(i+1)%2], sigma, ssarr[i%2]);
  }
  t0 = stop_watch(t0)/(double)niter;
  /* write the result after niter iterations */
  /* construct filename */
  sprintf(fname, "%s.ss%08d", argv[5], niter);
  /* convert from super-site packed */
  from_supersite(arr[0], ssarr[niter%2]);
  /* write to file */
  write_to_file(fname, arr[0]);
  /* write timing info */
  printf(" 1: iters = %8d, (Lx,Ly) = %6d, %6d, t = %8.1f usec/iter, BW = %6.3f GB/s, P = %6.3f Gflop/s\n",
	 niter, Lx, Ly, t0*1e6, 
	 Lx*Ly*sizeof(float)*2.0/(t0*1024*1024*1024), 
	 (Lx*Ly*6.0)/(t0*1.0e9));
  /* free super-site buffers */
  for(int i=0; i<2; i++) {
    free(ssarr[i]);
    free(arr[i]);
  }
  free(arr_in);
  return 0;
}
